package org.artifactory.ph;

import org.artifactory.common.resource.BaseResource;
import org.artifactory.core.api.ph.exception.PackageException;
import org.artifactory.core.api.ph.service.PackageUploadService;
import org.artifactory.factory.InfoFactoryHolder;
import org.artifactory.fs.MutableFileInfo;
import org.artifactory.repo.RepoPath;
import org.artifactory.repo.RepoPathFactory;
import org.artifactory.repo.SaveResourceContext;
import org.artifactory.repo.service.InternalRepositoryService;
import org.artifactory.repo.virtual.VirtualRepo;
import org.artifactory.resource.FileResource;

import java.io.InputStream;

import static org.apache.http.HttpStatus.SC_INTERNAL_SERVER_ERROR;
import static org.apache.http.HttpStatus.SC_NOT_FOUND;

public class PackageUploadServiceImpl implements PackageUploadService {

    private final InternalRepositoryService internalRepositoryService;

    public PackageUploadServiceImpl(InternalRepositoryService internalRepositoryService) {
        this.internalRepositoryService = internalRepositoryService;
    }

    @Override
    public void upload(String repoKey, String path, InputStream inputStream) {
        RepoPath repoPath = RepoPathFactory.create(repoKey, path);
        internalRepositoryService.deploy(repoPath, inputStream);
    }

    @Override
    public BaseResource uploadToVirtualCache(String repoKey, String path, InputStream inputStream)
            throws PackageException {
        VirtualRepo virtualRepo = internalRepositoryService.virtualRepositoryByKey(repoKey);
        if (virtualRepo == null) {
            throw new PackageException(SC_NOT_FOUND, "not found", "repository not found");
        }
        try {
            MutableFileInfo fileInfo = InfoFactoryHolder.get().createFileInfo(RepoPathFactory.create(repoKey, path));
            fileInfo.createTrustedChecksums();
            FileResource repoResource = new FileResource(fileInfo);
            SaveResourceContext saveContext = new SaveResourceContext.Builder(repoResource, inputStream).build();
            return internalRepositoryService.saveResourceInTransaction(virtualRepo, saveContext);
        } catch (Exception e) {
            throw new PackageException(SC_INTERNAL_SERVER_ERROR, "internal server error", e.getMessage(), e);
        }
    }
}
