package org.artifactory.repo.system.suppliers;

import lombok.extern.slf4j.Slf4j;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.descriptor.config.MutableCentralConfigDescriptor;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;

import static org.artifactory.descriptor.repo.RepoType.Generic;
import static org.artifactory.repo.config.RepoConfigDefaultValues.DEFAULT_REPO_LAYOUT;

/**
 * This is an abstract class which contains the template with which a system repo descriptor can be built
 *
 * @author Tomer Slutsky
 */
@Slf4j
public abstract class SystemRepoBase {
    final CentralConfigService centralConfigService;

    SystemRepoBase(CentralConfigService centralConfigService) {
        this.centralConfigService = centralConfigService;
    }

    public void addSystemRepo(MutableCentralConfigDescriptor mutableDescriptor, String repoKey) {
        centralConfigService.saveEditedDescriptorAndReload(mutableDescriptor,
                createSystemRepoDescriptor(mutableDescriptor, repoKey));
    }

    public final LocalRepoDescriptor createSystemRepoDescriptor(MutableCentralConfigDescriptor mutableDescriptor,
            String repoKey) {
        LocalRepoDescriptor repoDescriptor = buildDescriptor(repoKey);
        addToRepoList(mutableDescriptor, repoDescriptor);
        return repoDescriptor;
    }

    public LocalRepoDescriptor buildDescriptor(String repoKey) {
        LocalRepoDescriptor descriptor = new LocalRepoDescriptor();
        descriptor.setKey(repoKey);
        descriptor.setType(Generic);
        descriptor.setRepoLayout(centralConfigService.getDescriptor().getRepoLayout(DEFAULT_REPO_LAYOUT));
        return descriptor;
    }

    public void addToRepoList(MutableCentralConfigDescriptor mutableDescriptor,
            LocalRepoDescriptor descriptor) {
        mutableDescriptor.addLocalRepository(descriptor);
    }

}
