package org.artifactory.servicetrust;

import org.apache.commons.lang.NotImplementedException;
import org.artifactory.api.security.access.TokenSpec;
import org.artifactory.descriptor.authentication.AuthenticationToken;
import org.jetbrains.annotations.NotNull;
import org.jfrog.access.common.ServiceId;

import java.util.Optional;

import static org.artifactory.servicetrust.ServiceTrustService.INTERNAL_SCOPE_PREFIX;

/**
 * @author igoru
 */
public interface TokenSpecInboundService {

    /**
     * Create the specification for creating the paring token for this Artifactory instance
     */
    TokenSpec createPairingTokenSpec(ServiceId artifactoryServiceId, ServiceTrust trust, String subject, String scope,
            String baseUrl, String pairingUrl, String exchangeUrl);

    /**
     * Create the specification for creating the Master token for this Artifactory instance
     */
    TokenSpec createMasterTokenSpec(ServiceId artifactoryServiceId, String useCasePath, String pairingScope,
            String baseUrl, String pairingUrl, String exchangeUrl);

    @NotNull
    default String createRequiredInternalScope(@NotNull String useCasePath) {
        return INTERNAL_SCOPE_PREFIX + useCasePath;
    }

    /**
     * Create the specification for creating the scope token for this Artifactory instance
     */
    TokenSpec createExchangeTokenSpec(ServiceId artifactoryServiceId, String useCasePath);

    default void deleteAuthenticationToken(String baseUrl) {
        throw new NotImplementedException();
    }

    default Optional<AuthenticationToken> getAuthenticationToken(String baseUrl) {
        throw new NotImplementedException();
    }
}
