package org.artifactory.ph;

import org.artifactory.api.security.AuthorizationService;
import org.artifactory.api.security.SecurityService;
import org.artifactory.repo.RepoPath;
import org.artifactory.repo.RepoPathFactory;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.concurrent.Callable;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.reset;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.mockito.MockitoAnnotations.openMocks;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;

@Test
public class PackageSecurityServiceImplTest {

    @Mock
    private AuthorizationService authorizationService;
    @Mock
    private SecurityService securityService;

    private PackageSecurityServiceImpl packageSecurityService;

    @BeforeMethod
    public void setUp() {
        openMocks(this);

        packageSecurityService = new PackageSecurityServiceImpl(authorizationService, securityService);
    }

    @Test
    public void testCanRead() {
        when(authorizationService.canRead(any(RepoPath.class))).thenReturn(false);
        assertFalse(packageSecurityService.canRead("repo", "path"));
        verify(authorizationService, times(1)).canRead(eq(RepoPathFactory.create("repo", "path")));

        reset(authorizationService);

        when(authorizationService.canRead(any(RepoPath.class))).thenReturn(true);
        assertTrue(packageSecurityService.canRead("repo", "path"));
        verify(authorizationService, times(1)).canRead(eq(RepoPathFactory.create("repo", "path")));
    }

    @Test
    public void testCanWrite() {
        when(authorizationService.canDeploy(any(RepoPath.class))).thenReturn(false);
        assertFalse(packageSecurityService.canWrite("repo", "path"));
        verify(authorizationService, times(1)).canDeploy(eq(RepoPathFactory.create("repo", "path")));

        reset(authorizationService);

        when(authorizationService.canDeploy(any(RepoPath.class))).thenReturn(true);
        assertTrue(packageSecurityService.canWrite("repo", "path"));
        verify(authorizationService, times(1)).canDeploy(eq(RepoPathFactory.create("repo", "path")));
    }

    @Test
    public void testCanDelete() {
        when(authorizationService.canDelete(any(RepoPath.class))).thenReturn(false);
        assertFalse(packageSecurityService.canDelete("repo", "path"));
        verify(authorizationService, times(1)).canDelete(eq(RepoPathFactory.create("repo", "path")));

        reset(authorizationService);

        when(authorizationService.canDelete(any(RepoPath.class))).thenReturn(true);
        assertTrue(packageSecurityService.canDelete("repo", "path"));
        verify(authorizationService, times(1)).canDelete(eq(RepoPathFactory.create("repo", "path")));
    }

    @Test
    public void testCanAnnotate() {
        when(authorizationService.canAnnotate(any(RepoPath.class))).thenReturn(false);
        assertFalse(packageSecurityService.canAnnotate("repo", "path"));
        verify(authorizationService, times(1)).canAnnotate(eq(RepoPathFactory.create("repo", "path")));

        reset(authorizationService);

        when(authorizationService.canAnnotate(any(RepoPath.class))).thenReturn(true);
        assertTrue(packageSecurityService.canAnnotate("repo", "path"));
        verify(authorizationService, times(1)).canAnnotate(eq(RepoPathFactory.create("repo", "path")));
    }

    @Test
    public void testCanManage() {
        when(authorizationService.canManage(any(RepoPath.class))).thenReturn(false);
        assertFalse(packageSecurityService.canManage("repo", "path"));
        verify(authorizationService, times(1)).canManage(eq(RepoPathFactory.create("repo", "path")));

        reset(authorizationService);

        when(authorizationService.canManage(any(RepoPath.class))).thenReturn(true);
        assertTrue(packageSecurityService.canManage("repo", "path"));
        verify(authorizationService, times(1)).canManage(eq(RepoPathFactory.create("repo", "path")));
    }

    @Test
    public void testCallAsSystem() throws Exception {
        Mockito.doAnswer(invocationOnMock -> ((Callable) invocationOnMock.getArgument(0)).call())
                .when(securityService).callAsSystem(any(Callable.class));
        Callable<Boolean> callable = () -> true;
        assertTrue(securityService.callAsSystem(callable));
        verify(securityService, times(1)).callAsSystem(eq(callable));
    }
}