package org.artifactory.descriptor.repo;

import com.google.common.collect.Lists;
import org.artifactory.descriptor.Descriptor;
import org.jfrog.common.config.diff.DiffAtomic;
import org.jfrog.common.config.diff.DiffReference;
import org.jfrog.common.config.diff.GenerateDiffFunction;

import javax.validation.Valid;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlIDREF;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;
import java.util.List;
import java.util.Objects;

/**
 * @author Gidi Shabat
 */
@XmlType(name = "FederatedRepoType", propOrder = {"federatedMembers", "proxy",
        "modificationDate", "maxNumberOfThreads"},
        namespace = Descriptor.NS)
@GenerateDiffFunction
@XmlRootElement
public class FederatedRepoDescriptor extends LocalRepoDescriptor implements ProxyRepoDescriptor {

    @XmlElement(name = "federatedMember")
    @XmlElementWrapper(name = "federatedMembers")
    @DiffAtomic
    private List<FederatedMember> federatedMembers = Lists.newArrayList();

    @XmlIDREF
    @XmlElement(name = "proxyRef")
    @DiffReference
    @Valid
    private ProxyDescriptor proxy;

    private long modificationDate;

    @XmlElement
    private Integer maxNumberOfThreads;

    public Integer getMaxNumberOfThreads() {
        return maxNumberOfThreads;
    }

    public void setMaxNumberOfThreads(Integer maxNumberOfThreads) {
        this.maxNumberOfThreads = maxNumberOfThreads;
    }

    public List<FederatedMember> getFederatedMembers() {
        return federatedMembers;
    }

    public void setFederatedMembers(List<FederatedMember> federatedMembers) {
        this.federatedMembers = federatedMembers;
    }

    public long getModificationDate() {
        return modificationDate;
    }

    public void setModificationDate(long modificationDate) {
        this.modificationDate = modificationDate;
    }

    public ProxyDescriptor getProxy() {
        return proxy;
    }

    @Override
    public void setProxy(ProxyDescriptor proxy) {
        this.proxy = proxy;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof FederatedRepoDescriptor)) {
            return false;
        }
        FederatedRepoDescriptor that = (FederatedRepoDescriptor) o;
        if (!Objects.equals(federatedMembers, that.federatedMembers)) {
            return false;
        }
        if (!Objects.equals(proxy, that.proxy)) {
            return false;
        }
        return modificationDate == that.modificationDate;
    }

    @Override
    public int hashCode() {
        return Objects.hash(super.hashCode(), federatedMembers);
    }
}