package org.artifactory.core.api.ph.service;

import org.artifactory.core.api.ph.exception.PackageException;

import java.util.concurrent.Callable;

public interface PackageSecurityService {

    /**
     * Check read permission for artifact
     *
     * @param repoKey Repository key
     * @param path    Path to file
     * @return true if user can read file
     */
    boolean canRead(String repoKey, String path);

    /**
     * Check write permission for artifact
     *
     * @param repoKey Repository key
     * @param path    Path to file
     * @return true if user can write file
     */
    boolean canWrite(String repoKey, String path);

    /**
     * Check annotate permission for artifact
     *
     * @param repoKey Repository key
     * @param path    Path to file
     * @return true if user can annotate file
     */
    boolean canAnnotate(String repoKey, String path);

    /**
     * Check delete permission for artifact
     *
     * @param repoKey Repository key
     * @param path    Path to file
     * @return true if user can read file
     */
    boolean canDelete(String repoKey, String path);

    /**
     * Check manage permission for artifact
     *
     * @param repoKey Repository key
     * @param path    Path to file
     * @return true if user can manage file
     */
    boolean canManage(String repoKey, String path);

    /**
     * Do an action (given as {@link Callable}) as the system user. This method ensures to authenticate as the system
     * user before the action and restore the original authentication after.
     *
     * @param callable the action to perform
     */
    <V> V callAsSystem(Callable<V> callable) throws PackageException;
}
