package org.artifactory.storage.db.version;

import org.artifactory.storage.db.util.ArtifactoryDbVersionUtil;
import org.artifactory.version.CompoundVersionDetails;
import org.jfrog.converter.DbVersion;
import org.jfrog.converter.model.ConverterExecutionType;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

import static org.junit.jupiter.api.Assertions.assertTrue;

/**
 * @author Volodymyr Shulga
 */
public class ArtifactoryDBVersionTest {
    @Mock
    private CompoundVersionDetails compoundVersionDetails;
    private AutoCloseable closable;

    @BeforeEach
    public void setUp() {
        closable = MockitoAnnotations.openMocks(this);
    }

    @AfterEach
    public void close() throws Exception {
        if (closable != null) {
            closable.close();
        }
    }

    @Test
    void allVersionsReturnedWhenSrcAbsent() {
        final DbVersion[] dbVersions = ArtifactoryDBVersion.get(ConverterExecutionType.SEQ_SYNC);
        ArtifactoryDBVersion beforeVersion = ArtifactoryDBVersion.v32;
        Mockito.when(compoundVersionDetails.getVersion()).thenReturn(beforeVersion.applyFromVersion.get());
        for (DbVersion dbVersion : dbVersions) {
            ArtifactoryDBVersion artifactoryDBVersion = (ArtifactoryDBVersion) dbVersion;

            if (artifactoryDBVersion.isAlreadyApplied(compoundVersionDetails)) {
                assertTrue(ArtifactoryDbVersionUtil.isLessOrEqual(artifactoryDBVersion.version(), beforeVersion));
            } else {
                assertTrue(ArtifactoryDbVersionUtil.isGreaterOrEqual(artifactoryDBVersion.version(), beforeVersion));
            }
        }
    }

    @Test
    void shouldReturnConvertersInCorrectOrder() {
        final DbVersion[] dbVersions = ArtifactoryDBVersion.get(ConverterExecutionType.SEQ_SYNC);
        Assertions.assertEquals("v10", dbVersions[0].version());
        // verifies that sorting is correct and v11 goes after v10
        Assertions.assertEquals("v11", dbVersions[1].version());
    }
}
