package org.artifactory.rest.resource.system;

import org.artifactory.aol.AolService;
import org.artifactory.api.config.ImportService;
import org.artifactory.api.config.ImportSettingsImpl;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.config.ImportServiceImpl;
import org.artifactory.sapi.common.ImportSettings;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.core.Response;

import static org.apache.http.HttpStatus.SC_OK;
import static org.mockito.Mockito.any;
import static org.mockito.Mockito.doAnswer;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

/**
 * @author Alexei Vainshtein
 */
public class ImportResourceTest extends ArtifactoryHomeBoundTest {

    private ArtifactoryContext context;

    @BeforeMethod
    public void setup() {
        context = mock(ArtifactoryContext.class);
        ArtifactoryContextThreadBinder.bind(context);
    }

    @AfterMethod
    public void tearDown() {
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test(dataProvider = "actOnMissionControlProvider")
    public void testActivateImport(boolean shouldActOnMissionControl) {
        AolService aolService = mock(AolService.class);
        when(context.beanForType(AolService.class)).thenReturn(aolService);
        ImportSettingsConfigurationImpl settings = new ImportSettingsConfigurationImpl();
        settings.importMissionControl = shouldActOnMissionControl;
        settings.importPath = "path";

        ImportResource importResource = new ImportResource();
        ImportService importService = new ImportServiceImpl();
        importResource.importService = importService;
        doAnswer(invocationOnMock -> {
            ImportSettingsImpl argument = invocationOnMock.getArgument(0, ImportSettingsImpl.class);
            Assert.assertEquals(argument.shouldActOnMissionControl(), shouldActOnMissionControl);
            return null;
        }).when(context).importFrom(any(ImportSettings.class));
        importResource.httpResponse = mock(HttpServletResponse.class);
        Response response = importResource.activateImport(settings);
        Assert.assertEquals(response.getStatus(), SC_OK);
        verify(context, times(1)).importFrom(any(ImportSettings.class));
    }

    @DataProvider
    public Object[][] actOnMissionControlProvider() {
        return new Object[][] {
                {true},
                {false}
        };
    }
}